@@ -7,7 +7,7 @@ RDF::Trine::Node::Literal - RDF Node class for literals
 
 =head1 VERSION
 
-This document describes RDF::Trine::Node::Literal version 1.007
+This document describes RDF::Trine::Node::Literal version 1.011
 
 =cut
 
@@ -27,7 +27,7 @@ use Carp qw(carp croak confess);
 
 our ($VERSION, $USE_XMLLITERALS, $USE_FORMULAE);
 BEGIN {
-	$VERSION	= '1.007';
+	$VERSION	= '1.011';
 	eval "use RDF::Trine::Node::Literal::XML;";	## no critic (ProhibitStringyEval)
 	$USE_XMLLITERALS	= (RDF::Trine::Node::Literal::XML->can('new')) ? 1 : 0;
 	eval "use RDF::Trine::Node::Formula;";	## no critic (ProhibitStringyEval)
@@ -94,7 +94,20 @@ sub _new {
 	}
 	
 	if ($lang) {
-		$self	= [ $literal, lc($lang), undef ];
+		my $oldlang	= $lang;
+		# http://tools.ietf.org/html/bcp47#section-2.1.1
+		# All subtags use lowercase letters
+		$lang	= lc($lang);
+
+		# with 2 exceptions: subtags that neither appear at the start of the tag nor occur after singletons
+		# i.e. there's a subtag of length at least 2 preceding the exception; and a following subtag or end-of-tag
+
+		# 1. two-letter subtags are all uppercase
+		$lang	=~ s{(?<=\w\w-)(\w\w)(?=($|-))}{\U$1}g;
+
+		# 2. four-letter subtags are titlecase
+		$lang	=~ s{(?<=\w\w-)(\w\w\w\w)(?=($|-))}{\u\L$1}g;
+		$self	= [ $literal, $lang, undef ];
 	} elsif ($dt) {
 		if (blessed($dt)) {
 			$dt	= $dt->uri_value;
@@ -106,6 +119,7 @@ sub _new {
 	return bless($self, $class);
 }
 
+
 =item C<< literal_value >>
 
 Returns the string value of the literal.
