@@ -4,7 +4,7 @@ RDF::Trine::Store::Memory - Simple in-memory RDF store
 
 =head1 VERSION
 
-This document describes RDF::Trine::Store::Memory version 1.007
+This document describes RDF::Trine::Store::Memory version 1.011
 
 =head1 SYNOPSIS
 
@@ -39,7 +39,7 @@ use RDF::Trine::Error;
 my @pos_names;
 our $VERSION;
 BEGIN {
-	$VERSION	= "1.007";
+	$VERSION	= "1.011";
 	my $class	= __PACKAGE__;
 	$RDF::Trine::Store::STORE_CLASSES{ $class }	= $VERSION;
 	@pos_names	= qw(subject predicate object context);
@@ -616,7 +616,7 @@ change. This token is acceptable for use as an HTTP ETag.
 
 sub etag {
 	my $self	= shift;
-	return $self->{hash}->hexdigest;
+	return $self->{hash}->b64digest;
 }
 
 =item C<< size >>
